import { ROUTE_SPEC_FILE, DEFAULT_SERVER_CONFIG } from "@modern-js/utils";
import { genPluginImportsCode, getPluginsCode, severAppContextTemplate } from "../utils";
function genVercelEntry({ config, plugins, appContext } = {}) {
  const defaultConfig = {
    server: {
      port: 8080
    },
    output: {
      path: "."
    }
  };
  return `

  const fs = require('node:fs/promises');
  const path = require('node:path');
  const { createProdServer } = require('@modern-js/prod-server');
  ${genPluginImportsCode(plugins || [])}

  if(!process.env.NODE_ENV){
    process.env.NODE_ENV = 'production';
  }

  let requestHandler = null;
  async function createHandler() {
    try {
      let routes = [];
      const routeFilepath = path.join(__dirname, "${ROUTE_SPEC_FILE}");
      try {
        await fs.access(routeFilepath);
        const content = await fs.readFile(routeFilepath, "utf-8");
        const routeSpec = JSON.parse(content);
        routes = routeSpec.routes;
      } catch (error) {
        console.warn('route.json not found, continuing with empty routes.');
      }

      const prodServerOptions = {
        pwd: __dirname,
        routes,
        config: ${JSON.stringify(config || defaultConfig)},
        serverConfigFile: '${DEFAULT_SERVER_CONFIG}',
        plugins: ${getPluginsCode(plugins || [])},
        appContext: ${appContext ? severAppContextTemplate(appContext) : "undefined"},
        disableCustomHook: true
      }

      requestHandler = await createProdServer(prodServerOptions)

      return requestHandler;
    } catch(error) {
      console.error(error);
      process.exit(1);
    }
  }

  createHandler();

  module.exports = async(req, res) => {
    if(typeof requestHandler !== 'function'){
      await createHandler();
    }
    return requestHandler(req, res);
  }
  `;
}
export {
  genVercelEntry
};
