import path from "path";
import { fs as fse, getInternalPlugins } from "@modern-js/utils";
import { getProjectUsage } from "./utils";
import { handleDependencies } from "./dependencies";
var deploy_default = () => ({
  name: "@modern-js/plugin-deploy",
  pre: [
    "@modern-js/plugin-bff",
    "@modern-js/plugin-server"
  ],
  setup: (api) => {
    return {
      async beforeDeploy() {
        const deployTarget = process.env.MODERNJS_DEPLOY || "node";
        const appContext = api.useAppContext();
        const { appDirectory, distDirectory, serverInternalPlugins, sharedDirectory, apiDirectory, lambdaDirectory, metaName } = appContext;
        const configContext = api.useResolvedConfigContext();
        let outputDirectory = path.join(appDirectory, ".output");
        let funcsDirectory = outputDirectory;
        let staticDirectory = path.join(outputDirectory, "static");
        await fse.remove(outputDirectory);
        if (deployTarget === "node") {
          await fse.copy(distDirectory, outputDirectory, {
            filter: (src) => {
              const distStaticDirectory = path.join(distDirectory, "static");
              return !src.includes(distStaticDirectory);
            }
          });
        }
        if (deployTarget === "vercel") {
          outputDirectory = path.join(appDirectory, ".vercel/output");
          const config2 = {
            version: 3
          };
          await fse.ensureDir(outputDirectory);
          await fse.writeJSON(path.join(outputDirectory, "config.json"), config2);
          funcsDirectory = path.join(outputDirectory, "functions", "index.func");
          staticDirectory = path.join(outputDirectory, "static");
          await fse.ensureDir(funcsDirectory);
          await fse.copy(distDirectory, funcsDirectory, {
            filter: (src) => {
              const distStaticDirectory = path.join(distDirectory, "static");
              return !src.includes(distStaticDirectory);
            }
          });
          await fse.copy(path.join(distDirectory, "static"), staticDirectory);
          await fse.writeJSON(path.join(funcsDirectory, ".vc-config.json"), {
            runtime: "nodejs16.x",
            handler: "index.js",
            launcherType: "Nodejs",
            shouldAddHelpers: false,
            supportsResponseStreaming: true
          });
        }
        const { bff } = configContext;
        const config = {
          output: {
            path: "."
          },
          bff
        };
        const plugins = getInternalPlugins(appDirectory, serverInternalPlugins);
        const serverAppContext = {
          sharedDirectory: `path.join(__dirname, "${path.relative(appDirectory, sharedDirectory)}")`,
          apiDirectory: `path.join(__dirname, "${path.relative(appDirectory, apiDirectory)}")`,
          lambdaDirectory: `path.join(__dirname, "${path.relative(appDirectory, lambdaDirectory)}")`,
          metaName
        };
        let code = ``;
        console.log("deployTarget111111111", deployTarget);
        switch (deployTarget) {
          case "node": {
            const { genNodeEntry } = await import("./entrys/node");
            code = genNodeEntry({
              plugins,
              config,
              appContext: serverAppContext
            });
            break;
          }
          case "vercel": {
            const { genVercelEntry } = await import("./entrys/vercel");
            code = genVercelEntry({
              plugins,
              config,
              appContext: serverAppContext
            });
            break;
          }
          case "netlify": {
            const { genNetlifyEntry } = await import("./entrys/netlify");
            code = genNetlifyEntry({
              plugins,
              config,
              appContext: serverAppContext
            });
            break;
          }
          default: {
            code = `throw new Error("unknown deploy target, MODERNJS_DEPLOY should be set");`;
          }
        }
        const { useSSR, useAPI, useWebServer } = getProjectUsage(appDirectory, distDirectory);
        const entryFilePath = path.join(funcsDirectory, "index.js");
        if (useSSR || useAPI || useWebServer) {
          await fse.writeFile(entryFilePath, code);
        }
        await handleDependencies(appDirectory, funcsDirectory, [
          "@modern-js/prod-server"
        ]);
      }
    };
  }
});
export {
  deploy_default as default
};
