import { ROUTE_SPEC_FILE, DEFAULT_SERVER_CONFIG } from "@modern-js/utils";
import { genPluginImportsCode, getPluginsCode, severAppContextTemplate } from "../utils";
function genNetlifyEntry() {
  var _ref = arguments.length > 0 && arguments[0] !== void 0 ? arguments[0] : {}, config = _ref.config, plugins = _ref.plugins, appContext = _ref.appContext;
  var defaultConfig = {
    server: {
      port: 8080
    },
    output: {
      path: "."
    }
  };
  return "\n\n  const fs = require('node:fs/promises');\n  const path = require('node:path');\n  const { createNetlifyFunction } = require('@modern-js/prod-server/netlify');\n  ".concat(genPluginImportsCode(plugins || []), `

  let requestHandler = null;

  if(!process.env.NODE_ENV){
    process.env.NODE_ENV = 'production';
  }

  async function createHandler() {
    try {
      let routes = [];
      const routeFilepath = path.join(__dirname, "`).concat(ROUTE_SPEC_FILE, `");
      try {
        await fs.access(routeFilepath);
        const content = await fs.readFile(routeFilepath, "utf-8");
        const routeSpec = JSON.parse(content);
        routes = routeSpec.routes;
      } catch (error) {
        console.warn('route.json not found, continuing with empty routes.');
      }

      const prodServerOptions = {
        pwd: __dirname,
        routes,
        config: `).concat(JSON.stringify(config || defaultConfig), ",\n        serverConfigFile: '").concat(DEFAULT_SERVER_CONFIG, "',\n        plugins: ").concat(getPluginsCode(plugins || []), ",\n        appContext: ").concat(appContext ? severAppContextTemplate(appContext) : "undefined", ",\n        disableCustomHook: true\n      }\n\n      requestHandler = await createNetlifyFunction(prodServerOptions)\n\n      return requestHandler\n    } catch(error) {\n      console.error(error);\n      process.exit(1);\n    }\n  }\n\n  createHandler();\n\n  const handleRequest = async(request, context) => {\n    if(typeof requestHandler !== 'function'){\n      await createHandler();\n    }\n    return requestHandler(request, context);\n  }\n\n  export default handleRequest;\n  ");
}
export {
  genNetlifyEntry
};
